@@ -58,18 +58,17 @@ class ProxyConnection(Connection):
             self.close = self._proxy_connection_close
 
     def _proxy_connection_close(self):
-        if isinstance(self.http_conn, tuple) and len(self.http_conn) > 1:
-            conn = self.http_conn[1]
-            if hasattr(conn, 'close') and callable(conn.close):
-                conn.close()
-                self.http_conn = None
+        if self.http_conn:
+            self.http_conn[1].close()
+            self.http_conn = None
 
     def http_connection(self):
         def request_wrapper(fn):
             @wraps(fn)
-            def request_x_forwarded_for(method, url, body=None, headers=None):
+            def request_header_injection(method, url, body=None, headers=None):
                 if headers is None:
                     headers = {}
+                headers['Connection'] = 'close'
                 if self.real_ip:
                     headers['X-Forwarded-For'] = self.real_ip
                 if self.range_from:
@@ -81,7 +80,7 @@ class ProxyConnection(Connection):
                     fn(method, url, body=body, headers=headers)
                 else:  # swiftclient 2.0, ported to Requests
                     fn(method, url, data=body, headers=headers)
-            return request_x_forwarded_for
+            return request_header_injection
 
         parsed, conn = super(ProxyConnection, self).http_connection()
         conn.request = request_wrapper(conn.request)
@@ -134,6 +133,19 @@ def translate_objectstorage_error(fn):
             raise IOSError(err, msg)
     return wrapper
 
+def close_when_done(fn):
+    """
+    Decorator to close swift connection when the ftp command is done.
+    """
+    @wraps(fn)
+    def wrapper(obj, *args, **kwargs):
+        try:
+            return fn(obj, *args, **kwargs)
+        finally:
+            if obj.conn:
+                obj.close()
+    return wrapper
+
 def parse_fspath(path):
     """
     Returns a (container, path) tuple.
@@ -154,8 +166,8 @@ class ObjectStorageFD(object):
 
     split_size = 0
 
-    def __init__(self, cffs, container, obj, mode):
-        self.cffs = cffs
+    def __init__(self, connection, container, obj, mode):
+        self.conn = connection
         self.container = container
         self.name = obj
         self.mode = mode
@@ -192,11 +204,6 @@ class ObjectStorageFD(object):
     def part_name(self):
         return u"%s/%.6d" % (self.part_base_name, self.part)
 
-    @property
-    def conn(self):
-        """Connection to the storage."""
-        return self.cffs.conn
-
     def _start_copy_task(self):
         """
         Copy the first part of a multi-part file to its final location and create
@@ -224,6 +231,7 @@ class ObjectStorageFD(object):
                 logging.error("Failed to store the manifest %s: %s" % (name, ex.http_reason))
                 sys.exit(1)
             logging.debug("copy task done")
+            conn.close()
         self.pending_copy_task = multiprocessing.Process(target=copy_task,
                                                          args=(self.conn,
                                                                self.container,
@@ -271,18 +279,16 @@ class ObjectStorageFD(object):
     @translate_objectstorage_error
     def close(self):
         """Close the object and finish the data transfer."""
-        if 'r' in self.mode:
-            self.closed = True
-            self.conn.close()
-            return
-        if self.pending_copy_task:
-            logging.debug("waiting for a pending copy task...")
-            self.pending_copy_task.join()
-            logging.debug("wait is over")
-            if self.pending_copy_task.exitcode != 0:
-                raise IOSError(EIO, 'Failed to store the file')
-        if self.obj is not None:
-            self.obj.finish_chunk()
+        if 'r' not in self.mode:
+            if self.pending_copy_task:
+                logging.debug("waiting for a pending copy task...")
+                self.pending_copy_task.join()
+                logging.debug("wait is over")
+                if self.pending_copy_task.exitcode != 0:
+                    raise IOSError(EIO, 'Failed to store the file')
+            if self.obj is not None:
+                self.obj.finish_chunk()
+        self.obj = None
         self.closed = True
         self.conn.close()
 
@@ -298,9 +304,7 @@ class ObjectStorageFD(object):
             if self.total_size > 0:
                 self.conn.range_from = self.total_size
                 # we need to open a new connection to inject the `Range` header
-                if self.conn.http_conn:
-                    self.conn.http_conn[1].close()
-                    self.conn.http_conn = None
+                self.conn.close()
             _, self.obj = self.conn.get_object(self.container, self.name, resp_chunk_size=size)
 
         logging.debug("read size=%r, total_size=%r (range_from: %s)" % (size,
@@ -626,7 +630,7 @@ class ObjectStorageFS(object):
     memcache_hosts = None
 
     @translate_objectstorage_error
-    def __init__(self, username, api_key, authurl, keystone=None, hide_part_dir=False):
+    def __init__(self, username, api_key, authurl, keystone=None, hide_part_dir=False, snet=False):
         """
         Create the Object Storage connection.
 
@@ -634,11 +638,14 @@ class ObjectStorageFS(object):
         api_key
         authurl
         keystone - optional for auth 2.0 (keystone)
+        hider_part_dirt - optional, hide multipart .part files
+        snet - optional, use Rackspace's service network
         """
         self.conn = None
         self.authurl = authurl
         self.keystone = keystone
         self.hide_part_dir = hide_part_dir
+        self.snet = snet
         # A cache to hold the information from the last listdir
         self._listdir_cache = ListDirCache(self)
         self._cwd = '/'
@@ -651,7 +658,7 @@ class ObjectStorageFS(object):
         if not username or not api_key:
             raise ClientException("username/password required", http_status=401)
 
-        kwargs = dict(authurl=self.authurl, auth_version="1.0")
+        kwargs = dict(authurl=self.authurl, auth_version="1.0", snet=self.snet)
         tenant_name = None
 
         if self.keystone:
@@ -675,14 +682,16 @@ class ObjectStorageFS(object):
                                     )
         # force authentication
         self.conn.url, self.conn.token = self.conn.get_auth()
-        self.conn.http_conn = None
+        self.conn.close()
         # now we are authenticated and we have an username
         self.username = username
         self.tenant_name = tenant_name
 
     def close(self):
         """Explicitly close the connection, although it may not be required"""
-        self.conn.close()
+        logging.debug("called fs.close()")
+        if self.conn:
+            self.conn.close()
 
     def isabs(self, path):
         """Test whether a path is absolute"""
@@ -708,6 +717,7 @@ class ObjectStorageFS(object):
         logging.debug(e)
         raise IOSError(EPERM, 'Operation not permitted: %s' % e)
 
+    @close_when_done
     @translate_objectstorage_error
     def open(self, path, mode):
         """Open path with mode, raise IOError on error"""
@@ -715,7 +725,7 @@ class ObjectStorageFS(object):
         logging.debug("open %r mode %r" % (path, mode))
         self._listdir_cache.flush(posixpath.dirname(path))
         container, obj = parse_fspath(path)
-        return ObjectStorageFD(self, container, obj, mode)
+        return ObjectStorageFD(self.conn, container, obj, mode)
 
     def chdir(self, path):
         """Change current directory, raise OSError on error"""
@@ -746,6 +756,7 @@ class ObjectStorageFS(object):
             raise
         return True
 
+    @close_when_done
     @translate_objectstorage_error
     def mkdir(self, path):
         """
@@ -766,6 +777,7 @@ class ObjectStorageFS(object):
             logging.debug("Making container %r" % (container,))
             self.conn.put_container(container)
 
+    @close_when_done
     @translate_objectstorage_error
     def listdir(self, path):
         """
@@ -778,6 +790,7 @@ class ObjectStorageFS(object):
         list_dir = map(lambda x: unicode(x, 'utf-8'), self._listdir_cache.listdir(path))
         return list_dir
 
+    @close_when_done
     @translate_objectstorage_error
     def listdir_with_stat(self, path):
         """
@@ -789,6 +802,7 @@ class ObjectStorageFS(object):
         logging.debug("listdir_with_stat %r" % path)
         return [(unicode(name, 'utf-8)'), stat) for name, stat in self._listdir_cache.listdir_with_stat(path)]
 
+    @close_when_done
     @translate_objectstorage_error
     def rmdir(self, path):
         """
@@ -817,6 +831,7 @@ class ObjectStorageFS(object):
             logging.debug("Removing container %r" % (container,))
             self.conn.delete_container(container)
 
+    @close_when_done
     @translate_objectstorage_error
     def remove(self, path):
         """
@@ -856,6 +871,7 @@ class ObjectStorageFS(object):
         self.conn.put_container(dst_container_name)
         self._listdir_cache.flush("/")
 
+    @close_when_done
     @translate_objectstorage_error
     def rename(self, src, dst):
         """
@@ -991,6 +1007,7 @@ class ObjectStorageFS(object):
         except EnvironmentError:
             return False
 
+    @close_when_done
     @translate_objectstorage_error
     def stat(self, path):
         """
@@ -1042,6 +1059,7 @@ class ObjectStorageFS(object):
         logging.debug(e)
         raise IOSError(EPERM, 'Operation not permitted: %s' % e)
 
+    @close_when_done
     @translate_objectstorage_error
     def md5(self, path):
         """
